# Generated by roxygen2: do not edit by hand

S3method(print,calibration_results)
S3method(print,fitted_models)
S3method(print,model_projections)
S3method(print,prepared_data)
S3method(print,projection_data)
export(advanced_cleaning)
export(all_response_curves)
export(binarize_changes)
export(bivariate_response)
export(calibration)
export(colors_for_changes)
export(detect_concave)
export(explore_calibration_hist)
export(explore_partition_env)
export(explore_partition_extrapolation)
export(explore_partition_geo)
export(extract_occurrence_variables)
export(extract_var_from_formulas)
export(filter_decimal_precision)
export(fit_selected)
export(glm_mx)
export(glmnet_mx)
export(import_results)
export(independent_evaluation)
export(initial_cleaning)
export(move_2closest_cell)
export(organize_for_projection)
export(organize_future_worldclim)
export(partial_roc)
export(partition_response_curves)
export(perform_pca)
export(plot_calibration_hist)
export(plot_explore_partition)
export(plot_importance)
export(predict.glmnet_mx)
export(predict_selected)
export(prediction_changes)
export(prepare_data)
export(prepare_projection)
export(prepare_user_data)
export(project_selected)
export(projection_changes)
export(projection_mop)
export(projection_variability)
export(remove_cell_duplicates)
export(remove_corrdinates_00)
export(remove_duplicates)
export(remove_missing)
export(response_curve)
export(select_models)
export(single_mop)
export(sort_columns)
export(variable_importance)
importFrom(doSNOW,registerDoSNOW)
importFrom(enmpa,aux_var_comb)
importFrom(enmpa,predict_glm)
importFrom(foreach,`%dopar%`)
importFrom(foreach,foreach)
importFrom(fpROC,auc_metrics)
importFrom(glmnet,glmnet)
importFrom(glmnet,glmnet.control)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,filled.contour)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(mgcv,gam)
importFrom(mop,mop)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,deviance)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,predict.glm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(terra,NAflag)
importFrom(terra,app)
importFrom(terra,as.data.frame)
importFrom(terra,buffer)
importFrom(terra,cells)
importFrom(terra,clamp)
importFrom(terra,classify)
importFrom(terra,coltab)
importFrom(terra,crop)
importFrom(terra,diff)
importFrom(terra,distance)
importFrom(terra,ext)
importFrom(terra,extract)
importFrom(terra,is.factor)
importFrom(terra,levels)
importFrom(terra,mask)
importFrom(terra,mean)
importFrom(terra,median)
importFrom(terra,minmax)
importFrom(terra,ncell)
importFrom(terra,nlyr)
importFrom(terra,plot)
importFrom(terra,points)
importFrom(terra,prcomp)
importFrom(terra,predict)
importFrom(terra,rast)
importFrom(terra,rasterize)
importFrom(terra,res)
importFrom(terra,resample)
importFrom(terra,stdev)
importFrom(terra,trim)
importFrom(terra,unique)
importFrom(terra,unwrap)
importFrom(terra,vect)
importFrom(terra,wrap)
importFrom(terra,writeRaster)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
